--- **LibMouseGestures-1.0** Is a library for tracking and using mouse gestures
-- @class file
-- @name LibMouseGestures-1.0

--[[ LibMouseGestures-1.0

Revision: $Rev: 3 $
Author(s): Humbedooh
Description: Mouse gesture tracker
Dependencies: LibStub
License: MIT License

]]

--[===[@debug@
if ( dofile ) then
	dofile([[..\LibStub\LibStub.lua]]);
end
--@end-debug@]===]

local LIBMG = "LibMouseGestures-1.0"
local LIBMG_MINOR = tonumber(("$Rev: 03 $"):match("(%d+)")) or 10000;
if not LibStub then error(LIBMG .. " requires LibStub.") end
local LibMouseGestures = LibStub:NewLibrary(LIBMG, LIBMG_MINOR)
if not LibMouseGestures then return end

function LibMouseGestures:Setup(object)
	object.Update = LibMouseGestures.Update;
	object.UserInputReceived = LibMouseGestures.UserInputReceived;
	object.StartCapture = LibMouseGestures.StartCapture;
	object.GetBounds = LibMouseGestures.GetBounds;
	object.isRecording = false;
	object.mouseStart = {0,0};
	object.mouseEnd = {0,0};
	return object;
end

function LibMouseGestures:New(frame)
	if ( type(frame) == "table" and frame.Show ) then
		local MG = LibMouseGestures:Setup({});
		MG.drawLayer = CreateFrame("Frame", frame);
		MG.drawLayer:SetParent(frame);
		MG.drawLayer:SetAllPoints(frame);
		MG.drawLayer:EnableMouse();
		MG.drawLayer:SetScript("OnMouseDown", function(caller,event) MG:UserInputReceived(event,"Down") end);
		MG.drawLayer:SetScript("OnMouseUp", function(caller,event) MG:UserInputReceived(event,"Up") end);
		MG.drawLayer:SetScript("OnUpdate", function(caller,event) MG:Update() end);
		MG.drawLayer:Hide(); -- Initially hidden
		return MG;
	else
		error(LIBMG .. ": Usage: LibMG:New(target_frame).");
	end
end

function LibMouseGestures:StartCapture(script)
	local MG = self;
	if ( type(script) ~= "table" ) then return; end
	MG.CaptureScript = script;
	MG.drawLayer:SetFrameLevel(9999);
	MG.drawLayer:Raise();
	MG.drawLayer:Show();
end

function LibMouseGestures:Update()
	local MG = self;
	local scale = UIParent:GetEffectiveScale();
	local cursorX, cursorY = GetCursorPosition();
	local script = MG.CaptureScript or {};
	MG.mouseEnd = { (cursorX - (MG.drawLayer:GetLeft()*scale))/scale, ((MG.drawLayer:GetTop()*scale) - cursorY)/scale };
	if ( MG.isRecording ) then
		if ( MG.drawLayer:IsMouseOver() ) then
			if ( type(script.updateFunc) == "function" ) then
				script.updateFunc(MG, MG.mouseStart[1], MG.mouseStart[2], MG.mouseEnd[1], MG.mouseEnd[2]);
			end
		end
	end
	if ( script.tooltip ) then
		GameTooltip:ClearLines();
		GameTooltip:SetOwner(MG.drawLayer, "ANCHOR_TOPLEFT", MG.mouseEnd[1]+32, -MG.mouseEnd[2]-48);
		if ( type(script.tooltip) == "string" ) then
			GameTooltip:SetText(script.tooltip);
		elseif (type(script.tooltip) == "table") then
			GameTooltip:SetText(script.tooltip[1]);
			for n = 2, #script.tooltip do
				GameTooltip:AddLine(script.tooltip[n]);
			end
		end
		GameTooltip:Show();
	end
end


function LibMouseGestures:UserInputReceived(button,action)
	local MG = self;
	local script = MG.CaptureScript or {};
	if ( type(script) ~= "table" ) then return; end
	if ( not MG.isRecording ) then
		if ( script.startButton == (button .. action) ) then
			MG.isRecording = true;
			MG.mouseStart = { MG.mouseEnd[1], MG.mouseEnd[2] };
		elseif ( script.cancelButton == (button .. action) ) then
			MG.isRecording = false;
			if ( type(script.cancelFunc) == "function" ) then
				script.cancelFunc(MG, MG.mouseStart[1], MG.mouseStart[2], MG.mouseEnd[1], MG.mouseEnd[2]);
			end
			MG.CaptureScript = nil;
			GameTooltip:Hide();
			script = nil;
			MG.drawLayer:SetFrameLevel(0);
			MG.drawLayer:Hide();
		end
	else
		if ( script.stopButton == (button .. action) ) then
			MG.isRecording = false;
			if ( type(script.stopFunc) == "function" ) then
				script.stopFunc(MG, MG.mouseStart[1], MG.mouseStart[2], MG.mouseEnd[1], MG.mouseEnd[2]);
			end
			MG.CaptureScript = nil;
			GameTooltip:Hide();
			script = nil;
			MG.drawLayer:SetFrameLevel(0);
			MG.drawLayer:Hide();
		elseif ( script.cancelButton == (button .. action) ) then
			MG.isRecording = false;
			if ( type(script.cancelFunc) == "function" ) then
				script.cancelFunc(MG, MG.mouseStart[1], MG.mouseStart[2], MG.mouseEnd[1], MG.mouseEnd[2]);
			end
			MG.CaptureScript = nil;
			GameTooltip:Hide();
			script = nil;
			MG.drawLayer:SetFrameLevel(0);
			MG.drawLayer:Hide();
		elseif ( script.nextButton == (button .. action) ) then
			MG.mouseStart = { MG.mouseEnd[1], MG.mouseEnd[2]};
			if ( type(script.nextFunc) == "function" ) then
				script.nextFunc(MG, MG.mouseStart[1], MG.mouseStart[2], MG.mouseEnd[1], MG.mouseEnd[2]);
			end
		end
	end
end


function LibMouseGestures:GetBounds()
	local MG = self;
	local script = MG.CaptureScript or {};
	local sx,sy,ex,ey = MG.mouseStart[1], MG.mouseStart[2], MG.mouseEnd[1], MG.mouseEnd[2];
	local dx,dy = math.abs(sx-ex), math.abs(sy-ey);
	if ( dx < 5 ) then dx = 5; end
	if ( dy < 5 ) then dy = 5; end
	local a,b = math.min(sx,ex), math.min(sy,ey);
	if ( script.shiftModifier and IsShiftKeyDown() ) then
		dx,dy = math.min(dx,dy), math.min(dx,dy);
	end
	return a,b,dx,dy;
end
